@@ -19,10 +19,15 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 /*
-  $Id: ruli_conf.c,v 1.3 2004/05/21 18:23:50 evertonm Exp $
+  $Id: ruli_conf.c,v 1.6 2004/05/28 01:16:49 evertonm Exp $
  */
 
 
+#ifndef _GNU_SOURCE
+#define _GNU_SOURCE /* for GNU strtok_r(), inet_aton() */
+#endif
+
+
 #include <stdio.h>
 #include <assert.h>
 #include <string.h>
@@ -262,21 +267,25 @@ static int load_ns_list(ruli_list_t *ns_list)
 	 */
 	for (;;) {
 	  struct in_addr addr;
-	  struct in_addr *ad;
+	  struct in6_addr addr6;
+	  ruli_addr_t *ad = 0;
 	  
 	  tok = strtok_r(0, SEP, &ptr);
 	  if (!tok)
 	    break;
 
-	  if (!inet_aton(tok, &addr))
-	    break;
+	  /* IPv4 address? */
+	  if (inet_aton(tok, &addr))
+	    ad = ruli_addr_inet_new(addr);
+
+	  /* IPv6 address? */
+	  if (!ad)
+	    if (ruli_inet6_aton(tok, &addr6))
+	      ad = ruli_addr_inet6_new(addr6);
 
-	  ad = (struct in_addr *) ruli_malloc(sizeof(struct in_addr));
 	  if (!ad)
 	    break;
 
-	  *ad = addr;
-
 	  if (ruli_list_push(ns_list, ad))
 	    ruli_free(ad);
 	}
@@ -298,8 +307,8 @@ static int load_ns_list(ruli_list_t *ns_list)
             __FILE__, __PRETTY_FUNCTION__);
 
     for (i = 0; i < list_size; ++i) {
-      struct in_addr *addr = ruli_list_get(ns_list, i);
-      fprintf(stderr, " %s", inet_ntoa(*addr));
+      ruli_addr_t *addr = ruli_list_get(ns_list, i);
+      fprintf(stderr, " %s", inet_ntoa(addr->addr.ipv4));
     }
 
     fprintf(stderr, "\n");
