@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 /*
-  $Id: ruli_fsm.c,v 1.18 2004/05/21 18:23:50 evertonm Exp $
+  $Id: ruli_fsm.c,v 1.21 2004/05/31 05:57:40 evertonm Exp $
 
 
   TIMEOUT NOTES:
@@ -172,8 +172,11 @@ static void *on_udp_read_timeout(oop_source *oop_src,
 
 #ifdef RULI_RES_DEBUG
   fprintf(stderr, 
-	  "DEBUG: on_udp_read_timeout(): id=%d udp_readers=%d\n",
-	  qry->query_id, qry->resolver->udp_readers);
+	  "DEBUG: on_udp_read_timeout(): id=%d "
+	  "udp_readers=%d udp6_readers=%d\n",
+	  qry->query_id, 
+	  qry->resolver->udp_readers,
+	  qry->resolver->udp6_readers);
 #endif
 
   query_done_read_udp(qry);
@@ -252,10 +255,12 @@ static void *on_tcp_read_body_timeout(oop_source *oop_src,
 static void query_switch_status(ruli_res_query_t *qry, int status)
 {
 #ifdef RULI_RES_DEBUG
-  fprintf(stderr, "DEBUG: query_switch_status(id=%d index=%d): %d=>%d "
-	  "udp_writers=%d udp_readers=%d\n", 
+  fprintf(stderr, "DEBUG: query_switch_status(id=%d index=%d): %d=>%d"
+	  " udp_writers=%d udp_readers=%d"
+	  " udp6_writers=%d udp6_readers=%d\n", 
 	  qry->query_id, qry->resolver_index, qry->status, status,
-	  qry->resolver->udp_writers, qry->resolver->udp_readers);
+	  qry->resolver->udp_writers, qry->resolver->udp_readers,
+	  qry->resolver->udp6_writers, qry->resolver->udp6_readers);
 #endif
 
   qry->status = status;
@@ -265,20 +270,43 @@ void query_want_write_udp(ruli_res_query_t *qry)
 {
   ruli_res_t *res_ctx = qry->resolver;
   oop_source *oop_src = res_ctx->res_source;
-
+  ruli_addr_t *addr = ruli_res_get_curr_serv_addr(qry);
+  
   assert(qry->status == RULI_QRY_STAT_VOID);
-  assert(res_ctx->udp_writers >= 0);
 
   /*
-   * Only one query registers the sd for event monitoring.
+   * For each address family socket:
+   *
+   * Only one query registers the udp sd for event monitoring.
    * Many queries may later be waiting for write permission.
    * Thus we pass the resolver context to the on_udp_write event sink.
    * So any UDP_WANT_SEND-query can be selected in on_udp_write time.
    */
-  if (!res_ctx->udp_writers)
-    oop_src->on_fd(oop_src, res_ctx->udp_sd, OOP_WRITE, on_udp_write, res_ctx);
+
+  switch (addr->addr_family) {
+  case PF_INET:
+    assert(res_ctx->udp_writers >= 0);
+
+    if (!res_ctx->udp_writers)
+      oop_src->on_fd(oop_src, res_ctx->udp_sd, OOP_WRITE, on_udp_write,
+		     res_ctx);
   
-  ++res_ctx->udp_writers;
+    ++res_ctx->udp_writers;
+    break;
+
+  case PF_INET6:
+    assert(res_ctx->udp6_writers >= 0);
+
+    if (!res_ctx->udp6_writers)
+      oop_src->on_fd(oop_src, res_ctx->udp6_sd, OOP_WRITE, on_udp_write,
+		     res_ctx);
+  
+    ++res_ctx->udp6_writers;
+    break;
+
+  default:
+    assert(0);
+  }
   
   query_switch_status(qry, RULI_QRY_STAT_UDP_WANT_SEND);
 }
@@ -287,14 +315,34 @@ static void query_done_write_udp(ruli_res_query_t *qry)
 {
   ruli_res_t *res_ctx = qry->resolver;
   oop_source *oop_src = res_ctx->res_source;
+  ruli_addr_t *addr = ruli_res_get_curr_serv_addr(qry);
 
   assert(qry->status == RULI_QRY_STAT_UDP_WANT_SEND);
-  assert(res_ctx->udp_writers > 0);
 
-  --res_ctx->udp_writers;
+  switch (addr->addr_family) {
+  case PF_INET:
 
-  if (!res_ctx->udp_writers)
-    oop_src->cancel_fd(oop_src, res_ctx->udp_sd, OOP_WRITE);
+    assert(res_ctx->udp_writers > 0);
+
+    --res_ctx->udp_writers;
+    if (!res_ctx->udp_writers)
+      oop_src->cancel_fd(oop_src, res_ctx->udp_sd, OOP_WRITE);
+
+    break;
+
+  case PF_INET6:
+
+    assert(res_ctx->udp6_writers > 0);
+
+    --res_ctx->udp6_writers;
+    if (!res_ctx->udp6_writers)
+      oop_src->cancel_fd(oop_src, res_ctx->udp6_sd, OOP_WRITE);
+
+    break;
+
+  default:
+    assert(0);
+  }
 
   query_switch_status(qry, RULI_QRY_STAT_VOID);
 }
@@ -330,20 +378,45 @@ static void query_want_read_udp(ruli_res_query_t *qry)
 {
   ruli_res_t *res_ctx = qry->resolver;
   oop_source *oop_src = res_ctx->res_source;
+  ruli_addr_t *addr = ruli_res_get_curr_serv_addr(qry);
 
   assert(qry->status == RULI_QRY_STAT_VOID);
-  assert(res_ctx->udp_readers >= 0);
 
   /*
+   * For each socket address family:
+   *
    * Only one query registers the sd for event monitoring.
    * Many queries may later be waiting for write permission.
    * Thus we pass the resolver context to the on_udp_write event sink.
    * So any UDP_WANT_SEND-query can be selected in on_udp_write time.
    */
-  if (!res_ctx->udp_readers)
-    oop_src->on_fd(oop_src, res_ctx->udp_sd, OOP_READ, on_udp_read, res_ctx);
 
-  ++res_ctx->udp_readers;
+  switch (addr->addr_family) {
+  case PF_INET:
+
+    assert(res_ctx->udp_readers >= 0);
+
+    if (!res_ctx->udp_readers)
+      oop_src->on_fd(oop_src, res_ctx->udp_sd, OOP_READ, on_udp_read, 
+		     res_ctx);
+    ++res_ctx->udp_readers;
+
+    break;
+
+  case PF_INET6:
+
+    assert(res_ctx->udp6_readers >= 0);
+
+    if (!res_ctx->udp6_readers)
+      oop_src->on_fd(oop_src, res_ctx->udp6_sd, OOP_READ, on_udp_read, 
+		     res_ctx);
+    ++res_ctx->udp6_readers;
+
+    break;
+
+  default:
+    assert(0);
+  }
 
   /*
    * Set a timeout specific for this query.
@@ -359,19 +432,40 @@ static void query_done_read_udp(ruli_res_query_t *qry)
 {
   ruli_res_t *res_ctx = qry->resolver;
   oop_source *oop_src = res_ctx->res_source;
+  ruli_addr_t *addr = ruli_res_get_curr_serv_addr(qry);
 
   assert(qry->status == RULI_QRY_STAT_UDP_WANT_RECV);
-  assert(res_ctx->udp_readers > 0);
 
-  --res_ctx->udp_readers;
+  switch (addr->addr_family) {
+  case PF_INET:
 
-  /*
-   * Disable timeout
-   */
-  oop_src->cancel_time(oop_src, qry->tv, on_udp_read_timeout, qry);
+    assert(res_ctx->udp_readers > 0);
 
-  if (!res_ctx->udp_readers)
-    oop_src->cancel_fd(oop_src, res_ctx->udp_sd, OOP_READ);
+    --res_ctx->udp_readers;
+
+    oop_src->cancel_time(oop_src, qry->tv, on_udp_read_timeout, qry);
+    
+    if (!res_ctx->udp_readers)
+      oop_src->cancel_fd(oop_src, res_ctx->udp_sd, OOP_READ);
+    
+    break;
+
+  case PF_INET6:
+
+    assert(res_ctx->udp6_readers > 0);
+
+    --res_ctx->udp6_readers;
+
+    oop_src->cancel_time(oop_src, qry->tv, on_udp_read_timeout, qry);
+    
+    if (!res_ctx->udp6_readers)
+      oop_src->cancel_fd(oop_src, res_ctx->udp6_sd, OOP_READ);
+    
+    break;
+    
+  default:
+    assert(0);
+  }
 
   query_switch_status(qry, RULI_QRY_STAT_VOID);
 }
@@ -535,16 +629,16 @@ static void query_want_connect_tcp(ruli_res_query_t *qry)
    */
 
   for (;;) {
-    struct in_addr *server_addr = ruli_res_get_curr_serv_addr(qry);
-    int            server_port  = ruli_res_get_curr_serv_port(qry);
-    int            result;
+    ruli_addr_t *server_addr = ruli_res_get_curr_serv_addr(qry);
+    int         server_port  = ruli_res_get_curr_serv_port(qry);
+    int         result;
 
     server = ruli_res_get_curr_server(qry);
 
 #ifdef RULI_RES_DEBUG
     fprintf(stderr, 
 	    "DEBUG: query_want_connect_tcp(): trying %s:%d\n", 
-	    inet_ntoa(*server_addr), server_port);
+	    inet_ntoa(server_addr->addr.ipv4), server_port);
 #endif
 
     assert(qry->status == RULI_QRY_STAT_VOID);
@@ -560,7 +654,7 @@ static void query_want_connect_tcp(ruli_res_query_t *qry)
     /*
      * Create socket
      */
-    server->tcp_sd = ruli_sock_create_tcp();
+    server->tcp_sd = ruli_sock_create_tcp(server_addr->addr_family);
     if (server->tcp_sd == -1) {
 
       /*
@@ -692,13 +786,15 @@ static void query_done_connect_tcp(ruli_res_query_t *qry)
  * Note: As there is no timeout for the wanna-write
  *       condition, we don't need to disable the
  *       timeout here.
+ *
+ * udp_sd family is either PF_INET or PF_INET6.
  */
 static void *on_udp_write(oop_source *oop_src, int udp_sd, 
 			  oop_event event, void *ctx)
 {
   ruli_res_t       *res_ctx = (ruli_res_t *) ctx;
 
-  struct in_addr   *curr_server_addr;
+  ruli_addr_t      *curr_server_addr;
   int              curr_server_port;
   int              result;
 
@@ -832,17 +928,21 @@ static int check_answer_flags(ruli_uint16_t flags)
 
 #define UDP_READ_BUFSZ 2048
 
+/*
+  udp_sd family is either PF_INET or PF_INET6.
+*/
+
 static void *on_udp_read(oop_source *oop_src, int udp_sd, 
 			 oop_event event, void *ctx)
 {
-  ruli_res_t         *res_ctx = (ruli_res_t *) ctx;
-  ruli_uint8_t       buf[UDP_READ_BUFSZ];
-  struct sockaddr_in sa;
-  socklen_t          sa_len = sizeof(sa);
-  int                rd;
-  int                result;
-  ruli_res_query_t   *qry;
-  ruli_msg_header_t  msg_hdr;
+  ruli_res_t        *res_ctx = (ruli_res_t *) ctx;
+  ruli_uint8_t      buf[UDP_READ_BUFSZ];
+  struct sockaddr   sa;
+  socklen_t         sa_len = sizeof(sa);
+  int               rd;
+  int               result;
+  ruli_res_query_t  *qry;
+  ruli_msg_header_t msg_hdr;
 
   assert(event == OOP_READ);
 
@@ -898,8 +998,23 @@ static void *on_udp_read(oop_source *oop_src, int udp_sd,
   /* Query must be attached to right resolver */
   assert(qry->resolver == res_ctx);
 
+#ifndef NDEBUG
   /* Make sure the query was waiting on the proper socket */
-  assert(udp_sd == qry->resolver->udp_sd);
+  {
+    ruli_addr_t *addr = ruli_res_get_curr_serv_addr(qry);
+    
+    switch (addr->addr_family) {
+    case PF_INET:
+      assert(udp_sd == qry->resolver->udp_sd);
+      break;
+    case PF_INET6:
+      assert(udp_sd == qry->resolver->udp6_sd);
+      break;
+    default:
+      assert(0);
+    }
+  }
+#endif /* assert() NDEBUG */
 
   /*
    * Stop monitoring for reading and disable timeout
