@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 /*
-  $Id: ruli_parse.c,v 1.15 2004/05/21 18:23:50 evertonm Exp $
+  $Id: ruli_parse.c,v 1.16 2004/05/24 02:43:21 evertonm Exp $
   */
 
 
@@ -369,7 +369,7 @@ void ruli_parse_delete(ruli_parse_t *parse)
 int ruli_parse_rr_a(struct in_addr *addr,
 		    const ruli_uint8_t *rdata, ruli_uint16_t rdlength)
 {
-  int in_addr_size = sizeof(struct in_addr);
+  int in_addr_size = sizeof(*addr);
 
   if (rdlength != in_addr_size)
     return RULI_PARSE_RR_FAIL;
