@@ -19,10 +19,15 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 /*
-  $Id: ruli_res.c,v 1.26 2004/05/21 18:23:50 evertonm Exp $
+  $Id: ruli_res.c,v 1.29 2004/05/28 01:16:49 evertonm Exp $
   */
 
 
+#ifndef _GNU_SOURCE
+#define _GNU_SOURCE /* for GNU strncasecmp() */
+#endif
+
+
 #include <stdio.h>    /* FIXME: remove me [used for fprintf() debug] */
 
 #include <string.h>
@@ -237,6 +242,9 @@ int ruli_res_new(ruli_res_t *res_ctx)
   res_ctx->udp_sd        = -1;
   res_ctx->udp_writers   = 0;
   res_ctx->udp_readers   = 0;
+  res_ctx->udp6_sd       = -1;
+  res_ctx->udp6_writers  = 0;
+  res_ctx->udp6_readers  = 0;
   res_ctx->next_server   = 0;
   res_ctx->next_query_id = 0;
 
@@ -296,6 +304,15 @@ void ruli_res_delete(ruli_res_t *res_ctx)
     res_ctx->udp_sd = -1;
   }
 
+  /*
+   * Clean UDP6 socket
+   */
+  if (res_ctx->udp6_sd != -1) {
+    int result = close(res_ctx->udp6_sd);
+    assert(!result);
+    res_ctx->udp6_sd = -1;
+  }
+
   /*
    * Dispose servers information
    * Including TCP sockets
@@ -313,35 +330,53 @@ void ruli_res_delete(ruli_res_t *res_ctx)
 }
 
 /*
-  Many queries can use the same UDP socket.
+  Many queries can use the same UDP socket,
+  for a given address family.
   This function encapsulates this.
  */
-static int get_udp_socket(ruli_res_t *res_ctx)
+static int get_udp_socket(ruli_res_query_t *res_qry)
 {
-  int sd = res_ctx->udp_sd;
+  ruli_addr_t *addr = ruli_res_get_curr_serv_addr(res_qry);
+  ruli_res_t *res_ctx = res_qry->resolver;
+  int sd;
 
-  if (sd != -1)
-    return sd;
+  switch (addr->addr_family) {
+  case PF_INET:
+    sd = res_ctx->udp_sd;
+    if (sd != -1)
+      return sd;
+    sd = ruli_sock_create_udp(PF_INET);
+    if (sd == -1)
+      return sd;
+    res_ctx->udp_sd = sd;
+    break;
 
-  sd = ruli_sock_create_udp();
-  if (sd == -1)
-    return sd;
+  case PF_INET6:
+    sd = res_ctx->udp6_sd;
+    if (sd != -1)
+      return sd;
+    sd = ruli_sock_create_udp(PF_INET6);
+    if (sd == -1)
+      return sd;
+    res_ctx->udp6_sd = sd;
+    break;
 
-  res_ctx->udp_sd = sd;
+  default:
+    assert(0);
+  }
 
   return sd;
 }
 
 static int start_query(ruli_res_query_t *res_qry)
 {
-  ruli_res_t *res_ctx = res_qry->resolver;
   int result;
 
   /* 
    * Get UDP socket
    */
 
-  int udp_sd = get_udp_socket(res_ctx);
+  int udp_sd = get_udp_socket(res_qry);
   if (udp_sd == -1)
     return RULI_RES_SOCKET;
 
@@ -531,7 +566,7 @@ ruli_server_t *ruli_res_get_curr_server(ruli_res_query_t *qry)
 		                         ruli_res_get_curr_server_index(qry));
 }
 
-struct in_addr *ruli_res_get_curr_serv_addr(ruli_res_query_t *res_qry)
+ruli_addr_t *ruli_res_get_curr_serv_addr(ruli_res_query_t *res_qry)
 {
   ruli_res_t *res_ctx = res_qry->resolver;
 
@@ -542,7 +577,7 @@ struct in_addr *ruli_res_get_curr_serv_addr(ruli_res_query_t *res_qry)
   assert(res_qry->curr_server >= 0);
   assert(res_qry->curr_server < ruli_list_size(res_ctx->ns_list));
 
-  return (struct in_addr *) ruli_list_get(res_ctx->ns_list, res_qry->curr_server);
+  return (ruli_addr_t *) ruli_list_get(res_ctx->ns_list, res_qry->curr_server);
 }
 
 int ruli_res_get_curr_serv_port(ruli_res_query_t *res_qry)
