@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 /*
-  $Id: ruli_srv.c,v 1.56 2004/05/21 18:23:50 evertonm Exp $
+  $Id: ruli_srv.c,v 1.57 2004/05/24 02:43:21 evertonm Exp $
   */
 
 
@@ -238,7 +238,7 @@ static void *on_walk_answer(ruli_res_query_t *qry, void *qry_arg)
 	 */
 	{
 	  /* Allocate space */
-	  struct in_addr *addr = (struct in_addr *) ruli_malloc(sizeof(struct in_addr));
+	  ruli_addr_t *addr = (ruli_addr_t *) ruli_malloc(sizeof(*addr));
 	  if (!addr) {
 	    ruli_parse_delete(&parse);
 	    return walk_query_done(srv_qry, RULI_SRV_CODE_WALK_OTHER);
@@ -253,10 +253,13 @@ static void *on_walk_answer(ruli_res_query_t *qry, void *qry_arg)
 
 	  /* Write address into space */
 	  {
-	    int result = ruli_parse_rr_a(addr, rr->rdata, rr->rdlength);
+	    int result = ruli_parse_rr_a(&addr->addr.ipv4, rr->rdata, rr->rdlength);
 	    assert(!result); /* IN A parsing can't fail */
 	  }
-	}
+	  addr->addr_family = PF_INET;
+	  addr->addr_size = sizeof(addr->addr.ipv4);
+
+	} /* save address */
 
       } /* for */
     }
@@ -541,7 +544,7 @@ static void *on_fallback_answer(ruli_res_query_t *qry, void *qry_arg)
 	 * Save address
 	 */
 	{
-	  struct in_addr *addr;
+	  ruli_addr_t *addr;
 
 	  if (!srv_entry) {
 
@@ -581,7 +584,7 @@ static void *on_fallback_answer(ruli_res_query_t *qry, void *qry_arg)
 	  }
 
 	  /* Allocate space */
-	  addr = (struct in_addr *) ruli_malloc(sizeof(struct in_addr));
+	  addr = (ruli_addr_t *) ruli_malloc(sizeof(*addr));
 	  if (!addr) {
 	    ruli_parse_delete(&parse);
 	    return fall_query_done(srv_qry, RULI_SRV_CODE_FALL_OTHER);
@@ -596,10 +599,13 @@ static void *on_fallback_answer(ruli_res_query_t *qry, void *qry_arg)
 
 	  /* Write address into space */
 	  {
-	    int result = ruli_parse_rr_a(addr, rr->rdata, rr->rdlength);
+	    int result = ruli_parse_rr_a(&addr->addr.ipv4, rr->rdata, rr->rdlength);
 	    assert(!result); /* IN A parsing can't fail */
 	  }
-	}
+	  addr->addr_family = PF_INET;
+	  addr->addr_size = sizeof(addr->addr.ipv4);
+
+	} /* save address */
 
       } /* for: IN A scan of answer section */
 
@@ -1233,7 +1239,7 @@ static void *on_srv_answer(ruli_res_query_t *qry, void *arg)
 	/* Scan additional section */
 	for (j = 0; j < ad_list_size; ++j) {
 	  ruli_rr_t *rr = (ruli_rr_t *) ruli_list_get(ad_list, j);
-	  struct in_addr *addr;
+	  ruli_addr_t *addr;
 
 	  if (rr->qclass != RULI_RR_CLASS_IN)
 	    continue;
@@ -1250,7 +1256,7 @@ static void *on_srv_answer(ruli_res_query_t *qry, void *arg)
 	    continue;
 
 	  /* Allocate space for address */
-	  addr = (struct in_addr *) ruli_malloc(sizeof(struct in_addr));
+	  addr = (ruli_addr_t *) ruli_malloc(sizeof(*addr));
 	  if (!addr)
 	    return query_done(srv_qry, RULI_SRV_CODE_MALLOC);
 
@@ -1262,9 +1268,11 @@ static void *on_srv_answer(ruli_res_query_t *qry, void *arg)
 
 	  /* Write address into space */
 	  {
-	    int ret = ruli_parse_rr_a(addr, rr->rdata, rr->rdlength);
+	    int ret = ruli_parse_rr_a(&addr->addr.ipv4, rr->rdata, rr->rdlength);
 	    assert(!ret); /* IN A parsing can't fail */
 	  }
+	  addr->addr_family = PF_INET;
+	  addr->addr_size = sizeof(addr->addr.ipv4);
 
 	} /* for */
       }
@@ -1456,7 +1464,7 @@ void ruli_srv_query_delete(ruli_srv_t *srv_qry)
     int         i;
 
     /*
-     * For each entry, release list of *in_addr
+     * For each entry, release list of *ruli_addr_t
      */
     for (i = 0; i < list_size; ++i) {
       ruli_srv_entry_t *srv_entry = (ruli_srv_entry_t *) ruli_list_get(list, i);
