@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 /*
-  $Id: addrsolver.c,v 1.16 2004/05/19 23:09:12 evertonm Exp $
+  $Id: addrsolver.c,v 1.17 2004/05/24 02:43:21 evertonm Exp $
  */
 
 
@@ -143,8 +143,8 @@ static void *on_answer(ruli_res_query_t *qry, void *arg)
 
     size = ruli_list_size(&parse.answer_list);
     for (i = 0; i < size; ++i) {
-      struct in_addr addr;
-      ruli_rr_t      *rr;
+      ruli_addr_t addr;
+      ruli_rr_t   *rr;
 
       rr = ruli_list_get(&parse.answer_list, i);
 
@@ -154,7 +154,7 @@ static void *on_answer(ruli_res_query_t *qry, void *arg)
       if (rr->type != RULI_RR_TYPE_A)
 	continue;
 
-      result = ruli_parse_rr_a(&addr, rr->rdata, rr->rdlength);
+      result = ruli_parse_rr_a(&addr.addr.ipv4, rr->rdata, rr->rdlength);
       if (result)
 	continue;
 
@@ -182,7 +182,7 @@ static void *on_answer(ruli_res_query_t *qry, void *arg)
 	  continue;
       }
 
-      str_len = snprintf(buf, BUFSZ, "%s %s\n", domain, inet_ntoa(addr));
+      str_len = snprintf(buf, BUFSZ, "%s %s\n", domain, inet_ntoa(addr.addr.ipv4));
   
       assert(str_len < BUFSZ);
 
@@ -477,29 +477,33 @@ static void parse_servers(ruli_list_t *server_list, int serverc,
   assert(serverc >= 1);
 
   for (i = 0; i < serverc; ++i) {
-    struct in_addr *addr;
+    ruli_addr_t *addr;
 
-    addr = ruli_malloc(sizeof(struct in_addr));
+    addr = ruli_malloc(sizeof(*addr));
     if (!addr) {
       fprintf(stderr, "%s: can't allocate memory for address\n", prog_name);
       exit(1);
     }
 
-    if (!inet_aton(serverv[i], addr)) {
+    if (!inet_aton(serverv[i], &addr->addr.ipv4)) {
       fprintf(stderr, "%s: invalid IP address: %s\n", prog_name, serverv[i]);
       ruli_free(addr);
       exit(1);
     }
 
 #ifdef ADDRSOLVER_DEBUG
-    fprintf(stderr, "%s: saving server: %s\n", prog_name, inet_ntoa(*addr));
+    fprintf(stderr, "%s: saving server: %s\n", prog_name, inet_ntoa(addr->addr.ipv4));
 #endif
 
     {
       int result = ruli_list_push(server_list, addr);
       assert(!result);
     }
-  }
+    addr->addr_family = PF_INET;
+    addr->addr_size = sizeof(addr->addr.ipv4);
+
+  } /* for */
+
 }
 
 int main(int argc, const char **argv) 
@@ -511,7 +515,7 @@ int main(int argc, const char **argv)
   int         timeout;
   int         serverc;
   const char  **serverv;
-  ruli_list_t server_list; /* list of in_addr* */
+  ruli_list_t server_list; /* list of ruli_addr_t* */
 
   prog_name = argv[0];
 
