@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 /*
-  $Id: hostsolver.c,v 1.14 2004/05/19 23:09:13 evertonm Exp $
+  $Id: hostsolver.c,v 1.15 2004/05/24 02:43:21 evertonm Exp $
  */
 
 
@@ -190,8 +190,8 @@ static void *on_answer(ruli_res_query_t *qry, void *arg)
 
     size = ruli_list_size(&parse.answer_list);
     for (i = 0; i < size; ++i) {
-      struct in_addr addr;
-      ruli_rr_t      *rr;
+      ruli_addr_t addr;
+      ruli_rr_t   *rr;
 
       rr = ruli_list_get(&parse.answer_list, i);
 
@@ -201,10 +201,13 @@ static void *on_answer(ruli_res_query_t *qry, void *arg)
       if (rr->type != RULI_RR_TYPE_A)
 	continue;
 
-      result = ruli_parse_rr_a(&addr, rr->rdata, rr->rdlength);
+      result = ruli_parse_rr_a(&addr.addr.ipv4, rr->rdata, rr->rdlength);
       if (result)
 	continue;
 
+      addr.addr_family = PF_INET;
+      addr.addr_size = sizeof(addr.addr.ipv4);
+
       {
 	const int OWNER_BUFSZ = 256;
 	char      owner_buf[OWNER_BUFSZ];
@@ -229,7 +232,8 @@ static void *on_answer(ruli_res_query_t *qry, void *arg)
 	  continue;
       }
 
-      str_len = snprintf(buf, BUFSZ, "%s %s\n", domain, inet_ntoa(addr));
+      str_len = snprintf(buf, BUFSZ, "%s %s\n", domain, 
+			 inet_ntoa(addr.addr.ipv4));
   
       assert(str_len < BUFSZ);
 
@@ -305,10 +309,10 @@ static ruli_res_query_t *submit_query(ruli_res_t *res_ctx,
 #ifdef HOSTSOLVER_DEBUG
   fprintf(stderr, 
 	  "%s: DEBUG: submit_query(): domain=%s resolver=%u "
-	  "dname_buf=%u qry=%u dname_len=%d\n", 
+	  "dname_buf=%u qry=%u dname_len=%d class=%d type=%d\n", 
 	  prog_name, dom_str, (unsigned int) res_ctx,
 	  (unsigned int) dname_buf, (unsigned int) qry,
-	  dname_len);
+	  dname_len, qc, qt);
 #endif
 
   qry->q_on_answer     = on_answer;
@@ -584,29 +588,32 @@ static void parse_servers(ruli_list_t *server_list, int serverc,
   assert(serverc >= 1);
 
   for (i = 0; i < serverc; ++i) {
-    struct in_addr *addr;
+    ruli_addr_t *addr;
 
-    addr = ruli_malloc(sizeof(struct in_addr));
+    addr = ruli_malloc(sizeof(*addr));
     if (!addr) {
       fprintf(stderr, "%s: can't allocate memory for address\n", prog_name);
       exit(1);
     }
 
-    if (!inet_aton(serverv[i], addr)) {
+    if (!inet_aton(serverv[i], &addr->addr.ipv4)) {
       fprintf(stderr, "%s: invalid IP address: %s\n", prog_name, serverv[i]);
       ruli_free(addr);
       exit(1);
     }
 
 #ifdef HOSTSOLVER_DEBUG
-    fprintf(stderr, "%s: saving server: %s\n", prog_name, inet_ntoa(*addr));
+    fprintf(stderr, "%s: saving server: %s\n", prog_name, inet_ntoa(addr->addr.ipv4));
 #endif
 
     {
       int result = ruli_list_push(server_list, addr);
       assert(!result);
     }
-  }
+    addr->addr_family = PF_INET;
+    addr->addr_size = sizeof(addr->addr.ipv4);
+
+  } /* server for() loop */
 }
 
 int main(int argc, const char **argv) 
@@ -618,7 +625,7 @@ int main(int argc, const char **argv)
   int         timeout;
   int         serverc;
   const char  **serverv;
-  ruli_list_t server_list; /* list of in_addr* */
+  ruli_list_t server_list; /* list of ruli_addr_t* */
 
   prog_name = argv[0];
 
